@@ -111,7 +111,7 @@ struct Message msgtab[] = {
   { MSG_NICK,    m_nick,     0, MAXPARA, MSG_LAG, 0L},
   { MSG_NOTICE,  m_notice,   0, MAXPARA, MSG_LAG|MSG_REG, 0L},
   { MSG_JOIN,    m_join,     0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
-  { MSG_MODE,    m_mode,     0, MAXPARA, MSG_LAG|MSG_REG|MSG_PP, 0L},
+  { MSG_MODE,    m_mode,     0, MAXPARA, MSG_LAG|MSG_REG, 0L},
   { MSG_QUIT,    m_quit,     0, MAXPARA, MSG_LAG, 0L},
   { MSG_PART,    m_part,     0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
   { MSG_TOPIC,   m_topic,    0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
@@ -122,35 +122,33 @@ struct Message msgtab[] = {
   { MSG_PONG,    m_pong,     0, MAXPARA, MSG_LAG|MSG_REG, 0L},
   { MSG_ERROR,   m_error,    0, MAXPARA, MSG_LAG|MSG_REG|MSG_NOU, 0L},
 #ifdef	OPER_KILL
-  { MSG_KILL,    m_kill,     0, MAXPARA,
-				MSG_LAG|MSG_REG|MSG_OP|MSG_LOP, 0L},
+  { MSG_KILL,    m_kill,     0, MAXPARA, MSG_LAG|MSG_REG|MSG_OP|MSG_LOP, 0L},
 #else
   { MSG_KILL,    m_kill,     0, MAXPARA, MSG_LAG|MSG_REG|MSG_NOU, 0L},
 #endif
 #ifndef CLIENT_COMPILE
   { MSG_USER,    m_user,     0, MAXPARA, MSG_LAG|MSG_NOU, 0L},
   { MSG_AWAY,    m_away,     0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
-  { MSG_UMODE,   m_umode,    0, MAXPARA, MSG_LAG|MSG_REG, 0L},
+  { MSG_UMODE,   m_umode,    0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
   { MSG_ISON,    m_ison,     0, 1,	 MSG_LAG|MSG_REG, 0L},
   { MSG_SERVER,  m_server,   0, MAXPARA, MSG_LAG|MSG_NOU, 0L},
-  { MSG_SQUIT,   m_squit,    0, MAXPARA,
-				MSG_LAG|MSG_REG|MSG_OP|MSG_LOP, 0L},
-  { MSG_WHOIS,   m_whois,    0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
+  { MSG_SQUIT,   m_squit,    0, MAXPARA, MSG_LAG|MSG_REG|MSG_OP|MSG_LOP, 0L},
+  { MSG_WHOIS,   m_whois,    0, MAXPARA, MSG_LAG|MSG_REG, 0L},
   { MSG_WHO,     m_who,      0, MAXPARA, MSG_LAG|MSG_REG, 0L},
   { MSG_WHOWAS,  m_whowas,   0, MAXPARA, MSG_LAG|MSG_REG, 0L},
-  { MSG_LIST,    m_list,     0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
+  { MSG_LIST,    m_list,     0, MAXPARA, MSG_LAG|MSG_REG, 0L},
   { MSG_NAMES,   m_names,    0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
   { MSG_USERHOST,m_userhost, 0, MAXPARA, MSG_LAG|MSG_REG, 0L},
   { MSG_TRACE,   m_trace,    0, MAXPARA, MSG_LAG|MSG_REG, 0L},
   { MSG_PASS,    m_pass,     0, MAXPARA, MSG_LAG|MSG_NOU, 0L},
-  { MSG_LUSERS,  m_lusers,   0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
+  { MSG_LUSERS,  m_lusers,   0, MAXPARA, MSG_LAG|MSG_REG, 0L},
   { MSG_TIME,    m_time,     0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
   { MSG_OPER,    m_oper,     0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
   { MSG_CONNECT, m_connect,  0, MAXPARA,
 				MSG_LAG|MSG_REGU|MSG_OP|MSG_LOP, 0L},
   { MSG_VERSION, m_version,  0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
   { MSG_STATS,   m_stats,    0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
-  { MSG_LINKS,   m_links,    0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
+  { MSG_LINKS,   m_links,    0, MAXPARA, MSG_LAG|MSG_REG, 0L},
   { MSG_ADMIN,   m_admin,    0, MAXPARA, MSG_LAG, 0L},
   { MSG_USERS,   m_users,    0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
   { MSG_SUMMON,  m_summon,   0, MAXPARA, MSG_LAG|MSG_REGU, 0L},
