@@ -123,6 +123,7 @@
 #define	RPL_YOURHOST         002
 #define	RPL_CREATED          003
 #define	RPL_MYINFO           004
+#define	RPL_BOUNCE           005
 
 /*
  * Errors are in the range from 400-599 currently and are grouped by what
@@ -137,7 +138,6 @@
 #define ERR_TOOMANYTARGETS   407
 #define ERR_NOSUCHSERVICE    408
 #define	ERR_NOORIGIN         409
-#define	ERR_TOOMANYDESTS     410
 
 #define ERR_NORECIPIENT      411
 #define ERR_NOTEXTTOSEND     412
@@ -299,13 +299,15 @@
 #define RPL_SERVLIST         234
 #define RPL_SERVLISTEND      235
 
+#define	RPL_STATSVLINE       240
 #define	RPL_STATSLLINE       241
 #define	RPL_STATSUPTIME      242
 #define	RPL_STATSOLINE       243
 #define	RPL_STATSHLINE       244
 #define	RPL_STATSSLINE       245
 #define	RPL_STATSPING        246
-#define RPL_STATSDEFINE      248
+#define	RPL_STATSBLINE       247
+#define	RPL_STATSDEFINE      248
 #define	RPL_STATSDEBUG	     249
 
 #define	RPL_LUSERCLIENT      251
