@@ -55,7 +55,7 @@
 
 /*
  *	@(#)resolv.h	8.1 (Berkeley) 6/2/93
- *	$Id: resolv.h,v 1.1.1.1 1997/04/14 13:25:03 kalt Exp $
+ *	$Id: resolv.h,v 1.2 1997/05/15 20:31:38 kalt Exp $
  */
 
 #ifndef _RESOLV_H_
@@ -71,14 +71,14 @@
 #include <stdio.h>
 
 /*
- * revision information.  this is the release date in YYYYMMDD format.
- * it can change every day so the right thing to do with it is use it
- * in preprocessor commands such as "#if (__RES > 19931104)".  do not
+ * Revision information.  This is the release date in YYYYMMDD format.
+ * It can change every day so the right thing to do with it is use it
+ * in preprocessor commands such as "#if (__RES > 19931104)".  Do not
  * compare for equality; rather, use it to determine whether your resolver
  * is new enough to contain a certain feature.
  */
 
-#define	__RES	19951031
+#define	__RES	19960801
 
 /*
  * Resolver configuration file.
@@ -110,9 +110,9 @@ struct __res_state {
 	struct sockaddr_in
 		nsaddr_list[MAXNS];	/* address of name server */
 #define	nsaddr	nsaddr_list[0]		/* for backward compatibility */
-	u_short	id;			/* current packet id */
+	u_short	id;			/* current message id */
 	char	*dnsrch[MAXDNSRCH+1];	/* components of domain to search */
-	char	defdname[MAXDNAME];	/* default domain */
+	char	defdname[256];		/* default domain (deprecated) */
 	u_long	pfcode;			/* RES_PRF_ flags - see below. */
 	unsigned ndots:4;		/* threshold for initial abs. query */
 	unsigned nsort:4;		/* number of elements in sort_list[] */
@@ -121,7 +121,7 @@ struct __res_state {
 		struct in_addr	addr;
 		u_int32_t	mask;
 	} sort_list[MAXRESOLVSORT];
-	char	pad[72];		/* On an i38this means 512b total. */
+	char	pad[72];		/* on an i386 this means 512b total */
 };
 
 /*
@@ -140,6 +140,7 @@ struct __res_state {
 #define	RES_INSECURE1	0x00000400	/* type 1 security disabled */
 #define	RES_INSECURE2	0x00000800	/* type 2 security disabled */
 #define	RES_NOALIASES	0x00001000	/* shuts off HOSTALIASES feature */
+#define	RES_USE_INET6	0x00002000	/* use/map IPv6 in gethostbyname() */
 
 #define RES_DEFAULT	(RES_RECURSE | RES_DEFNAMES | RES_DNSRCH)
 
@@ -181,10 +182,30 @@ typedef res_sendhookact (*res_send_rhook)__P((const struct sockaddr_in *ns,
 					      int anssiz,
 					      int *resplen));
 
+struct res_sym {
+	int	number;		/* Identifying number, like T_MX */
+	char *	name;		/* Its symbolic name, like "MX" */
+	char *	humanname;	/* Its fun name, like "mail exchanger" */
+};
+
 extern struct __res_state ircd_res;
+extern const struct res_sym __p_class_syms[];
+extern const struct res_sym __p_type_syms[];
 
 /* Private routines shared between libc/net, named, nslookup and others. */
+#define	res_hnok	__res_hnok
+#define	res_ownok	__res_ownok
+#define	res_mailok	__res_mailok
+#define	res_dnok	__res_dnok
+#define	sym_ston	__sym_ston
+#define	sym_ntos	__sym_ntos
+#define	sym_ntop	__sym_ntop
+#define b64_ntop	__b64_ntop
+#define	b64_pton	__b64_pton
+#define	loc_ntoa	__loc_ntoa
+#define	loc_aton	__loc_aton
 #define	dn_skipname	__dn_skipname
+#define	fp_resstat	__fp_resstat
 #define	fp_query	__fp_query
 #define	fp_nquery	__fp_nquery
 #define	hostalias	__hostalias
@@ -193,52 +214,83 @@ extern struct __res_state ircd_res;
 #define p_class		__p_class
 #define p_time		__p_time
 #define p_type		__p_type
+#define	p_query		__p_query
 #define	p_cdnname	__p_cdnname
 #define	p_cdname	__p_cdname
+#define	p_fqnname	__p_fqnname
 #define	p_fqname	__p_fqname
 #define	p_rr		__p_rr
 #define	p_option	__p_option
+#define	p_secstodate	__p_secstodate
+#define	dn_count_labels	__dn_count_labels
+#define	dn_comp		__dn_comp
 #define	res_randomid	__res_randomid
+#define	res_send	__res_send
 #define	res_isourserver	__res_isourserver
 #define	res_nameinquery	__res_nameinquery
 #define	res_queriesmatch __res_queriesmatch
+#define	res_close	__res_close
+
+#ifdef BIND_RES_POSIX3
+#define	dn_expand	__dn_expand
+#define	res_init	__res_init
+#define	res_query	__res_query
+#define	res_search	__res_search
+#define	res_querydomain	__res_querydomain
+#define	res_mkquery	__res_mkquery
+#endif
+
 __BEGIN_DECLS
-int	 __dn_skipname __P((const u_char *, const u_char *));
-void	 __fp_resstat __P((struct __res_state *, FILE *));
-void	 __fp_query __P((const u_char *, FILE *));
-void	 __fp_nquery __P((const u_char *, int, FILE *));
-char	*__hostalias __P((const char *));
-void	 __putlong __P((u_int32_t, u_char *));
-void	 __putshort __P((u_int16_t, u_char *));
-char	*__p_time __P((u_int32_t));
-void	 __p_query __P((const u_char *));
-const u_char *__p_cdnname __P((const u_char *, const u_char *, int, FILE *));
-const u_char *__p_cdname __P((const u_char *, const u_char *, FILE *));
-const u_char *__p_fqname __P((const u_char *, const u_char *, FILE *));
-const u_char *__p_rr __P((const u_char *, const u_char *, FILE *));
-const char *__p_type __P((int));
-const char *__p_class __P((int));
-const char *__p_option __P((u_long option));
-int	 dn_comp __P((const char *, u_char *, int, u_char **, u_char **));
-int	 dn_expand __P((const u_char *, const u_char *, const u_char *,
-			char *, int));
-int	 res_init __P((void));
-u_int16_t res_randomid __P((void));
-int	 res_query __P((const char *, int, int, u_char *, int));
-int	 res_search __P((const char *, int, int, u_char *, int));
-int	 res_querydomain __P((const char *, const char *, int, int,
-			      u_char *, int));
-int	 res_mkquery __P((int, const char *, int, int, const u_char *, int,
-			  const u_char *, u_char *, int));
-int	 res_send __P((const u_char *, int, u_char *, int));
-int	 res_isourserver __P((const struct sockaddr_in *));
-int	 res_nameinquery __P((const char *, int, int,
-			      const u_char *, const u_char *));
-int	 res_queriesmatch __P((const u_char *, const u_char *,
-			       const u_char *, const u_char *));
-/* XXX - these last two don't belong in the resolver */
-u_int	 inet_nsap_addr __P((const char *, u_char *, int maxlen));
-char	*inet_nsap_ntoa __P((int, const u_char *, char *ascii));
+int		res_hnok __P((const char *));
+int		res_ownok __P((const char *));
+int		res_mailok __P((const char *));
+int		res_dnok __P((const char *));
+int		sym_ston __P((const struct res_sym *, char *, int *));
+const char *	sym_ntos __P((const struct res_sym *, int, int *));
+const char *	sym_ntop __P((const struct res_sym *, int, int *));
+int		b64_ntop __P((u_char const *, size_t, char *, size_t));
+int		b64_pton __P((char const *, u_char *, size_t));
+int		loc_aton __P((const char *, u_char *));
+const char *	loc_ntoa __P((const u_char *, char *));
+int		__ircd_dn_skipname __P((const u_char *, const u_char *));
+void		fp_resstat __P((struct __res_state *, FILE *));
+void		fp_query __P((const u_char *, FILE *));
+void		fp_nquery __P((const u_char *, int, FILE *));
+const char *	hostalias __P((const char *));
+void		putlong __P((u_int32_t, u_char *));
+void		putshort __P((u_int16_t, u_char *));
+const char *	p_class __P((int));
+const char *	p_time __P((u_int32_t));
+const char *	p_type __P((int));
+void		p_query __P((const u_char *));
+const u_char *	p_cdnname __P((const u_char *, const u_char *, int, FILE *));
+const u_char *	p_cdname __P((const u_char *, const u_char *, FILE *));
+const u_char *	p_fqnname __P((const u_char *cp, const u_char *msg,
+			       int, char *, int));
+const u_char *	p_fqname __P((const u_char *, const u_char *, FILE *));
+const u_char *	p_rr __P((const u_char *, const u_char *, FILE *));
+const char *	p_option __P((u_long option));
+char *		p_secstodate __P((u_long));
+int		dn_count_labels __P((char *));
+int		ircd_dn_comp __P((const char *, u_char *, int,
+			     u_char **, u_char **));
+int		ircd_dn_expand __P((const u_char *, const u_char *, const u_char *,
+			       char *, int));
+int		ircd_res_init __P((void));
+u_int		ircd_res_randomid __P((void));
+int		res_query __P((const char *, int, int, u_char *, int));
+int		res_search __P((const char *, int, int, u_char *, int));
+int		res_querydomain __P((const char *, const char *, int, int,
+				     u_char *, int));
+int		ircd_res_mkquery __P((int, const char *, int, int, const u_char *, int,
+				 const u_char *, u_char *, int));
+int		res_send __P((const u_char *, int, u_char *, int));
+int		res_isourserver __P((const struct sockaddr_in *));
+int		res_nameinquery __P((const char *, int, int,
+				     const u_char *, const u_char *));
+int		res_queriesmatch __P((const u_char *, const u_char *,
+				      const u_char *, const u_char *));
+void		res_close __P((void));
 __END_DECLS
 
 #endif /* !_RESOLV_H_ */
