@@ -17,7 +17,7 @@
  *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #ifndef lint
-static char sccsid[] = "@(#)hash.c	1.1 1/21/95 (C) 1991 Darren Reed";
+static  char rcsid[] = "@(#)$Id: hash.c,v 1.3 1997/06/26 15:40:44 kalt Exp $";
 #endif
 
 #include "struct.h"
@@ -31,6 +31,7 @@ static char sccsid[] = "@(#)hash.c	1.1 1/21/95 (C) 1991 Darren Reed";
 static	aHashEntry	*clientTable = NULL;
 static	aHashEntry	*channelTable = NULL;
 static	aHashEntry	*serverTable = NULL;
+static	unsigned int	*hashtab = NULL;
 static	int	clhits = 0, clmiss = 0, clsize = 0;
 static	int	chhits = 0, chmiss = 0, chsize = 0;
 static	int	svsize = 0;
@@ -73,18 +74,18 @@ int	_SERVERSIZE = 0;
  * or division or modulus in the inner loop.  subtraction and other bit
  * operations allowed.
  */
-int	hash_nick_name(nname, store)
+static	u_int	hash_nick_name(nname, store)
 char	*nname;
 int	*store;
 {
 	Reg	u_char	*name = (u_char *)nname;
 	Reg	u_char	ch;
-	Reg	int	hash = 1;
+	Reg	u_int	hash = 1;
 
 	for (; (ch = *name); name++)
 	{
 		hash <<= 1;
-		hash += tolower(ch) * 109;
+		hash += hashtab[(int)ch];
 	}
 	*store = hash;
 	if (hash < 0)
@@ -101,23 +102,21 @@ int	*store;
  * is little or no point hashing on a full channel name which maybe 255 chars
  * long.
  */
-int	hash_channel_name(hname, store)
+static	u_int	hash_channel_name(hname, store)
 char	*hname;
 int	*store;
 {
 	Reg	u_char	*name = (u_char *)hname;
 	Reg	u_char	ch;
 	Reg	int	i = 30;
-	Reg	int	hash = 5;
+	Reg	u_int	hash = 5;
 
 	for (; (ch = *name) && --i; name++)
 	{
 		hash <<= 1;
-		hash += tolower(ch) * 109 + (i << 1);
+		hash += hashtab[(u_int)ch] + (i << 1);
 	}
 	*store = hash;
-	if (hash < 0)
-		hash = -hash;
 	hash %= _CHANNELHASHSIZE;
 	return (hash);
 }
@@ -204,10 +203,21 @@ int	size;
 
 void	inithashtables()
 {
+	Reg int i;
+
 	clear_client_hash_table((_HASHSIZE) ? _HASHSIZE : HASHSIZE);
 	clear_channel_hash_table((_CHANNELHASHSIZE) ? _CHANNELHASHSIZE
                                  : CHANNELHASHSIZE);
 	clear_server_hash_table((_SERVERSIZE) ? _SERVERSIZE : SERVERSIZE);
+
+	/*
+	 * Moved multiplication out from the hashfunctions and into
+	 * a pre-generated lookup table. Should save some CPU usage
+	 * even on machines with a fast mathprocessor.  -- Core
+	 */
+	hashtab = (u_int *) MyMalloc(256 * sizeof(u_int));
+	for (i = 0; i < 256; i++)
+		hashtab[i] = tolower((char)i) * 109;
 }
 
 static	void	bigger_hash_table(size, table, new)
@@ -293,7 +303,7 @@ int	add_to_client_hash_table(name, cptr)
 char	*name;
 aClient	*cptr;
 {
-	Reg	int	hashv;
+	Reg	u_int	hashv;
 
 	hashv = hash_nick_name(name, &cptr->hashv);
 	cptr->hnext = (aClient *)clientTable[hashv].list;
@@ -313,7 +323,7 @@ int	add_to_channel_hash_table(name, chptr)
 char	*name;
 aChannel	*chptr;
 {
-	Reg	int	hashv;
+	Reg	u_int	hashv;
 
 	hashv = hash_channel_name(name, &chptr->hashv);
 	chptr->hnextch = (aChannel *)channelTable[hashv].list;
@@ -333,7 +343,7 @@ int	add_to_server_hash_table(sptr, cptr)
 aServer	*sptr;
 aClient	*cptr;
 {
-	Reg	int	hashv;
+	Reg	u_int	hashv;
 
 	Debug((DEBUG_DEBUG, "Add %s token %d/%d/%s cptr %#x to server table",
 		sptr->bcptr->name, sptr->stok, sptr->ltok, sptr->tok, cptr));
@@ -357,11 +367,9 @@ char	*name;
 aClient	*cptr;
 {
 	Reg	aClient	*tmp, *prev = NULL;
-	Reg	int	hashv;
+	Reg	u_int	hashv;
 
 	hashv = cptr->hashv;
-	if (hashv < 0)
-		hashv = -hashv;
 	hashv %= _HASHSIZE;
 	for (tmp = (aClient *)clientTable[hashv].list; tmp; tmp = tmp->hnext)
 	    {
@@ -403,11 +411,9 @@ char	*name;
 aChannel	*chptr;
 {
 	Reg	aChannel	*tmp, *prev = NULL;
-	Reg	int	hashv;
+	Reg	u_int	hashv;
 
 	hashv = chptr->hashv;
-	if (hashv < 0)
-		hashv = -hashv;
 	hashv %= _CHANNELHASHSIZE;
 	for (tmp = (aChannel *)channelTable[hashv].list; tmp;
 	     tmp = tmp->hnextch)
@@ -445,11 +451,9 @@ aServer	*sptr;
 aClient	*cptr;
 {
 	Reg	aServer	*tmp, *prev = NULL;
-	Reg	int	hashv;
+	Reg	u_int	hashv;
 
 	hashv = sptr->stok * 15053;
-	if (hashv < 0)
-		hashv = -hashv;
 	hashv %= _SERVERSIZE;
 	for (tmp = (aServer *)serverTable[hashv].list; tmp; tmp = tmp->shnext)
 	    {
@@ -488,7 +492,7 @@ aClient	*cptr;
 	Reg	aClient	*tmp;
 	Reg	aClient	*prv = NULL;
 	Reg	aHashEntry	*tmp3;
-	int	hashv, hv;
+	u_int	hashv, hv;
 
 	hashv = hash_nick_name(name, &hv);
 	tmp3 = &clientTable[hashv];
@@ -524,29 +528,6 @@ aClient	*cptr;
 	return (cptr);
 }
 
-/*
- * hash_find_nickserv
- */
-aClient	*hash_find_nickserv(name, cptr)
-char	*name;
-aClient *cptr;
-{
-	aClient	*c2ptr = cptr;
-	char	*serv;
-
-	if ((serv = (char *)index(name, '@')))
-	    {
-		*serv++ = '\0';
-		c2ptr = hash_find_server(serv, cptr);
-		if (c2ptr && IsMe(c2ptr))
-			c2ptr = hash_find_client(name, cptr);
-		*--serv = '@';
-	    }
-	else
-		c2ptr = hash_find_client(name, cptr);
-	return (c2ptr);
-}
-
 /*
  * hash_find_server
  */
@@ -558,7 +539,7 @@ aClient *cptr;
 	Reg	char	*t;
 	Reg	char	ch;
 	aHashEntry	*tmp3;
-	int	hashv, hv;
+	u_int	hashv, hv;
 
 	hashv = hash_nick_name(server, &hv);
 	tmp3 = &clientTable[hashv];
@@ -587,7 +568,7 @@ aClient *cptr;
 	 * Whats happening in this next loop ? Well, it takes a name like
 	 * foo.bar.edu and proceeds to earch for *.edu and then *.bar.edu.
 	 * This is for checking full server names against masks although
-	 * it isnt often done this way in lieu of using matches().
+	 * it isnt often done this way in lieu of using match().
 	 */
 	for (;;)
 	    {
@@ -623,7 +604,7 @@ aChannel *chptr;
 {
 	Reg	aChannel	*tmp, *prv = NULL;
 	Reg	aHashEntry	*tmp3;
-	int	hashv, hv;
+	u_int	hashv, hv;
 
 	hashv = hash_channel_name(name, &hv);
 	tmp3 = &channelTable[hashv];
@@ -657,7 +638,7 @@ void	*dummy;
 {
 	Reg	aServer	*tmp, *prv = NULL;
 	Reg	aHashEntry	*tmp3;
-	int	hashv, hv;
+	u_int	hashv, hv;
 
 	hv = hashv = tok * 15053;
 	hashv %= _SERVERSIZE;
@@ -773,7 +754,7 @@ char	*parv[];
 		for (i = 0; i < 10; i++)
 			sendto_one(sptr,"NOTICE %s :Entires with %d links : %d",
 			parv[0], i, link_pop[i]);
-		return (1);
+		return (2);
 	case 'r' :
 	    {
 		Reg	aClient	*acptr;
@@ -799,20 +780,20 @@ char	*parv[];
 			sendto_one(sptr,"NOTICE %s :%s hash to entry %d",
 				   parv[0], parv[2],
 				   hash_channel_name(parv[2]));
-		return (1);
+		return (2);
 	case 'h' :
 		if (parc > 2)
 			sendto_one(sptr,"NOTICE %s :%s hash to entry %d",
 				   parv[0], parv[2],
 				   hash_nick_name(parv[2]));
-		return (1);
+		return (2);
 	case 'n' :
 	    {
 		aClient	*tmp;
 		int	max;
 
-		if (parc <= 2)
-			return (0);
+		if (parc <= 2 || !IsAnOper(sptr))
+			return (1);
 		l = atoi(parv[2]) % _HASHSIZE;
 		if (parc > 3)
 			max = atoi(parv[3]) % _HASHSIZE;
@@ -827,15 +808,15 @@ char	*parv[];
 				sendto_one(sptr,"NOTICE %s :Node: %d #%d %s",
 					   parv[0], l, i, tmp->name);
 			    }
-		return (1);
+		return (2);
 	    }
 	case 'N' :
 	    {
 		aChannel *tmp;
 		int	max;
 
-		if (parc <= 2)
-			return (0);
+		if (parc <= 2 || !IsAnOper(sptr))
+			return (1);
 		l = atoi(parv[2]) % _CHANNELHASHSIZE;
 		if (parc > 3)
 			max = atoi(parv[3]) % _CHANNELHASHSIZE;
@@ -846,7 +827,7 @@ char	*parv[];
 			     i++, tmp = tmp->hnextch)
 				sendto_one(sptr,"NOTICE %s :Node: %d #%d %s",
 					   parv[0], l, i, tmp->chname);
-		return (1);
+		return (2);
 	    }
 	case 'S' :
 #else
@@ -860,26 +841,26 @@ char	*parv[];
 #ifndef	DEBUGMODE
 	}
 #endif
-	    return 1;
+	    return 2;
 #ifdef	DEBUGMODE
 	case 'z' :
 	    {
-		if (parc <= 2)
-			return 0;
+		if (parc <= 2 || !IsAnOper(sptr))
+			return 1;
 		l = atoi(parv[2]);
 		if (l < 256)
-			return 0;
+			return 1;
 		bigger_hash_table(&_HASHSIZE, clientTable, l);
 		sendto_one(sptr, "NOTICE %s :HASHSIZE now %d", parv[0], l);
 		break;
 	    }
 	case 'Z' :
 	    {
-		if (parc <= 2)
-			return 0;
+		if (parc <= 2 || !IsAnOper(sptr))
+			return 1;
 		l = atoi(parv[2]);
 		if (l < 256)
-			return 0;
+			return 1;
 		bigger_hash_table(&_CHANNELHASHSIZE, channelTable, l);
 		sendto_one(sptr, "NOTICE %s :CHANNELHASHSIZE now %d",
 			   parv[0], l);
@@ -908,6 +889,6 @@ char	*parv[];
 		   parv[0], clhits, clmiss);
 	sendto_one(sptr,"NOTICE %s :Channel hits %d miss %d",
 		   parv[0], chhits, chmiss);
-	return 1;
+	return 2;
 #endif
 }
