@@ -22,8 +22,7 @@
  */
 
 #ifndef lint
-static  char sccsid[] = "%W% %G% (C) 1988 University of Oulu, \
-Computing Center and Jarkko Oikarinen";
+static  char rcsid[] = "@(#)$Id: s_serv.c,v 1.18 1997/07/16 19:27:16 kalt Exp $";
 #endif
 
 #include <sys/types.h>
@@ -41,6 +40,8 @@ Computing Center and Jarkko Oikarinen";
 #include <fcntl.h>
 #include "h.h"
 
+extern  char    serveropts[];
+
 static	char	buf[BUFSIZE];
 
 /*
@@ -110,15 +111,10 @@ aClient *sptr, *cptr;
 int	parc;
 char	*parv[];
 {
-	extern	char	serveropts[];
-
 	if (hunt_server(cptr,sptr,":%s VERSION :%s",1,parc,parv)==HUNTED_ISME)
-	    {
 		sendto_one(sptr, rpl_str(RPL_VERSION, parv[0]),
 			   version, debugmode, ME, serveropts);
-		return 1;
-	    }
-	return 1;
+	return 2;
 }
 
 /*
@@ -212,6 +208,11 @@ char	*parv[];
 		sendto_one(sptr, err_str(ERR_NOSUCHSERVER, parv[0]), server);
 		return 1;
 	    }
+	if (MyConnect(sptr) && !MyConnect(acptr) && parc < 3)
+	    {
+                sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS,parv[0]), "SQUIT");
+		return 0;
+            }
 	if (IsLocOp(sptr) && !MyConnect(acptr))
 	    {
 		sendto_one(sptr, err_str(ERR_NOPRIVILEGES, parv[0]));
@@ -248,18 +249,49 @@ char	*parv[];
     }
 
 /*
-** check_version (Vesa)
-**      ver = new server version string
+** check_version
+**      The PASS command delivers additional information about incoming
+**	connection. The data is temporarily stored to info/name/username
+**	in m_pass() and processed here before the fields are natively used.
+** Return: < 1: exit/error, > 0: no error
 */
-int	check_version(ver)
-char	*ver;
+int	check_version(cptr)
+aClient	*cptr;
 {
-	Debug((DEBUG_INFO,"check_version: %s", ver));
-	if (!strncmp(ver, "0209", 4))
-		return SV_29;	/* We have something special here! */
-	if (!strncmp(ver, "021", 3))
-		return SV_29;	/* for future versions */
-	return SV_OLD;
+	Debug((DEBUG_INFO,"check_version: %s, %s, %s", cptr->info,
+	       cptr->info+12, cptr->info+44));
+
+	/* hop = 1 really for local client, return it in m_server_estab() */
+	if (!strncmp(cptr->info, "0209", 4) || !strncmp(cptr->info, "021", 3))
+		cptr->hopcount = SV_29;	/* 2.9+ protocol */
+	else
+		cptr->hopcount = SV_OLD;
+
+	if (*cptr->info) {	/* Check version number/mask from conf */
+		if (find_two_masks(cptr->name, cptr->info, CONF_VER)) {
+			sendto_flag(SCH_ERROR, "Bad version %s from %s",
+				    cptr->info, get_client_name(cptr, TRUE));
+			return exit_client(cptr, cptr, &me, "Bad version");
+		}
+	} else
+		return 1;	/* No version checked */
+
+	strncpyzt(buf, cptr->info+12, 30);
+	if (*buf) {	/* Check version flags from conf */
+		if (find_conf_flags(cptr->name, buf, CONF_VER)) {
+			sendto_flag(SCH_ERROR, "Bad flags %s from %s",
+				    buf, get_client_name(cptr, TRUE));
+			return exit_client(cptr, cptr, &me, "Bad flags");
+		}
+	} else
+		return 2;	/* No flags checked */
+
+	/* right now, I can't code anything good for this */
+	/* Stop whining, and do it! ;) */
+	if (strchr(cptr->info+44, 'Z'))	/* Compression requested */
+                cptr->flags |= FLAGS_ZIPRQ;
+
+	return 3;
 }
 
 /*
@@ -280,16 +312,15 @@ char	*parv[];
 	char	info[REALLEN+1], *inpath, *host, *stok;
 	aClient *acptr, *bcptr;
 	aConfItem *aconf;
-	int	hop, token = 0;
+	int	hop = 0, token = 0;
 
 	info[0] = info[REALLEN] = '\0';	/* strncpy() doesn't guarantee NULL */
 	inpath = get_client_name(cptr, FALSE);
 	if (parc < 2 || *parv[1] == '\0')
 	    {
 			sendto_one(cptr,"ERROR :No servername");
-			return 2;
+			return 1;
 	    }
-	hop = 0;
 	host = parv[1];
 	if (parc > 3 && (hop = atoi(parv[2])))
 	    {
@@ -301,9 +332,9 @@ char	*parv[];
 	else if (parc > 2)
 	    {
 		(void)strncpy(info, parv[2], REALLEN);
-		if (parc > 3)
+		i = strlen(info);
+		if (parc > 3 && ((i+2) < REALLEN))
 		    {
-				i = strlen(info);
 				(void)strncat(info, " ", REALLEN - i - 1);
 				(void)strncat(info, parv[3], REALLEN - i - 2);
 		    }
@@ -343,29 +374,29 @@ char	*parv[];
 		** Rather than KILL the link which introduced it, KILL the
 		** youngest of the two links. -avalon
 		*/
-                bcptr = (cptr->firsttime > acptr->from->firsttime) ? cptr :
-                        acptr->from;
-                sendto_one(bcptr, "ERROR :Server %s already exists", host);
-                /* in both cases the bcptr (the youngest is killed) */
-                if (bcptr == cptr)
-                    {   
-                        sendto_flag(SCH_ERROR,
+		bcptr = (cptr->firsttime > acptr->from->firsttime) ? cptr :
+			acptr->from;
+		sendto_one(bcptr, "ERROR :Server %s already exists", host);
+		/* in both cases the bcptr (the youngest is killed) */
+		if (bcptr == cptr)
+		    {
+			sendto_flag(SCH_ERROR,
 			    "Link %s cancelled, server %s already exists",
-                                    get_client_name(bcptr, TRUE), host);
-                        return exit_client(bcptr, bcptr, &me, "Server Exists");
-                    }
-                else
-                    {   
-                        /*
-                        ** in this case, we are not dropping the link from
-                        ** which we got the SERVER message.  Thus we canNOT
-                        ** `return' yet! -krys
-                        */
-                        sendto_flag(SCH_ERROR,
+				    get_client_name(bcptr, TRUE), host);
+			return exit_client(bcptr, bcptr, &me, "Server Exists");
+		    }
+		else
+		    {
+			/*
+			** in this case, we are not dropping the link from
+			** which we got the SERVER message.  Thus we canNOT
+			** `return' yet! -krys
+			*/
+			strcpy(buf, get_client_name(bcptr, TRUE));
+			sendto_flag(SCH_ERROR,
 			    "Link %s cancelled, server %s reintroduced by %s",
-                                    get_client_name(bcptr, TRUE), host,
-                                    get_client_name(cptr, TRUE));
-                        (void) exit_client(bcptr, bcptr, &me, "Server Exists");
+				    buf, host, get_client_name(cptr, TRUE));
+			(void) exit_client(bcptr, bcptr, &me, "Server Exists");
 		    }
 	    }
 	if ((acptr = find_person(host, NULL)) && (acptr != cptr))
@@ -390,6 +421,19 @@ char	*parv[];
 
 	if (IsServer(cptr))
 	    {
+		/* A server can only be introduced by another server. */
+		if (!IsServer(sptr))
+		    {
+			sendto_flag(SCH_LOCAL,
+			    "Squitting %s brought by %s (introduced by %s)",
+				    host, get_client_name(cptr, FALSE),
+				    sptr->name);
+			sendto_one(cptr,
+				   ":%s SQUIT %s :(Introduced by %s from %s)",
+				   me.name, host, sptr->name,
+				   get_client_name(cptr, FALSE));
+	  		return 1;
+		    }
 		/*
 		** Server is informing about a new server behind
 		** this link. Create REMOTE server structure,
@@ -403,7 +447,7 @@ char	*parv[];
 				   host);
 			sendto_flag(SCH_ERROR, "No server info for %s from %s",
 				    host, get_client_name(cptr, TRUE));
-	  		return 0;
+	  		return 1;
 		    }
 
 		/*
@@ -468,9 +512,7 @@ char	*parv[];
 		strncpyzt(acptr->info, info, sizeof(acptr->info));
 		acptr->serv->up = sptr->name;
 		acptr->serv->stok = token;
-#ifdef KRYS
 		acptr->serv->snum = find_server_num(acptr->name);
-#endif
 		SetServer(acptr);
 		istat.is_serv++;
 		add_client_to_list(acptr);
@@ -494,7 +536,7 @@ char	*parv[];
 				return exit_client(cptr, cptr, &me,
 						   "Lost N line");
 			    }
-			if (matches(my_name_for_link(ME, aconf->port),
+			if (match(my_name_for_link(ME, aconf->port),
 				    acptr->name) == 0)
 				continue;
 			stok = (bcptr->serv->version != SV_OLD) 
@@ -503,9 +545,10 @@ char	*parv[];
 				   acptr->name, hop+1, stok, acptr->info);
 		    }
 #ifdef	USE_SERVICES
-		check_services_butone(SERVICE_WANT_SERVER, acptr->name, sptr,
-					":%s SERVER %s %d :%s", parv[0],
-					acptr->name, hop+1, acptr->info);
+		check_services_butone(SERVICE_WANT_SERVER, acptr->name, acptr,
+				      ":%s SERVER %s %d %s :%s", parv[0],
+				      acptr->name, hop+1, acptr->serv->tok,
+				      acptr->info);
 #endif
 		sendto_flag(SCH_SERVER, "Received SERVER %s from %s (%d %s)",
 			    acptr->name, parv[0], hop+1, acptr->info);
@@ -514,15 +557,17 @@ char	*parv[];
 
 	if ((!IsUnknown(cptr) && !IsHandshake(cptr)) ||
 	    (cptr->flags & FLAGS_UNKCMD))
-		return 0;
+		return 1;
 	/*
 	** A local link that is still in undefined state wants
 	** to be a SERVER. Check if this is allowed and change
 	** status accordingly...
 	*/
-	/* hop = 1 really for local client, return it in m_server_estab() */
-	cptr->hopcount = check_version(cptr->info);
 	strncpyzt(cptr->name, host, sizeof(cptr->name));
+	/* cptr->name has to exist before check_version(), and cptr->info
+	 * may not be filled before check_version(). */
+	if ((hop = check_version(cptr)) < 1)
+		return hop;	/* from exit_client() */
 	strncpyzt(cptr->info, info[0] ? info:ME, sizeof(cptr->info));
 
 	switch (check_server_init(cptr))
@@ -530,7 +575,7 @@ char	*parv[];
 	case 0 :
 		return m_server_estab(cptr);
 	case 1 :
-		sendto_flag(SCH_NOTICE, "Checking access for %s ",
+		sendto_flag(SCH_NOTICE, "Checking access for %s",
 			    get_client_name(cptr,TRUE));
 		return 1;
 	default :
@@ -563,7 +608,8 @@ Reg	aClient	*cptr;
 			    "Access denied. No N line for server %s", inpath);
 		return exit_client(cptr, cptr, &me, "No N line for server");
 	    }
-	if (!(bconf = find_conf(cptr->confs, host, CONF_CONNECT_SERVER)))
+	if (!(bconf = find_conf(cptr->confs, host, CONF_CONNECT_SERVER|
+				CONF_ZCONNECT_SERVER)))
 	    {
 		ircstp->is_ref++;
 		sendto_one(cptr, "ERROR :Only N (no C) field for server %s",
@@ -627,8 +673,14 @@ Reg	aClient	*cptr;
 	if (IsUnknown(cptr))
 	    {
 		if (bconf->passwd[0])
-			sendto_one(cptr,"PASS %s %s", bconf->passwd,
-				   pass_version);
+#ifndef	ZIP_LINKS
+			sendto_one(cptr, "PASS %s %s %s", bconf->passwd,
+				   pass_version, serveropts);
+#else
+			sendto_one(cptr, "PASS %s %s %s %s", bconf->passwd,
+				   pass_version, serveropts,
+			   (bconf->status == CONF_ZCONNECT_SERVER) ? "Z" : "");
+#endif
 		/*
 		** Pass my info to the new server
 		*/
@@ -662,7 +714,7 @@ Reg	aClient	*cptr;
 		*s = '\0'; /* should never be NULL */
 		Debug((DEBUG_INFO, "Check Usernames [%s]vs[%s]",
 			aconf->host, cptr->username));
-		if (matches(aconf->host, cptr->username))
+		if (match(aconf->host, cptr->username))
 		    {
 			*s = '@';
 			ircstp->is_ref++;
@@ -676,6 +728,23 @@ Reg	aClient	*cptr;
 		*s = '@';
 	    }
 
+#ifdef	ZIP_LINKS
+	if ((cptr->flags & FLAGS_ZIPRQ) &&
+	    (bconf->status == CONF_ZCONNECT_SERVER))	    
+	    {
+		if (zip_init(cptr) == -1)
+		    {
+			zip_free(cptr);
+			sendto_flag(SCH_ERROR,
+			    "Unable to setup compressed link for %s",
+				    get_client_name(cptr, TRUE));
+			return exit_client(cptr, cptr, &me,
+					   "zip_init() failed");
+		    }
+		cptr->flags |= FLAGS_ZIP;
+	    }
+#endif
+
 	det_confs_butmask(cptr, CONF_LEAF|CONF_HUB|CONF_NOCONNECT_SERVER);
 	/*
 	** *WARNING*
@@ -694,8 +763,8 @@ Reg	aClient	*cptr;
 	istat.is_serv++;
 	istat.is_myserv++;
 	nextping = timeofday;
-	sendto_flag(SCH_NOTICE, "Link with %s established. (%d)", inpath,
-		    cptr->hopcount);
+	sendto_flag(SCH_NOTICE, "Link with %s established. (%d%s)", inpath,
+		    cptr->hopcount, (cptr->flags & FLAGS_ZIP) ? "z" : "");
 	(void)add_to_client_hash_table(cptr->name, cptr);
 	/* doesnt duplicate cptr->serv if allocted this struct already */
 	(void)make_server(cptr);
@@ -703,9 +772,7 @@ Reg	aClient	*cptr;
 	cptr->serv->nline = aconf;
 	cptr->serv->version = cptr->hopcount;   /* temporary location */
 	cptr->hopcount = 1;			/* local server connection */
-#ifdef KRYS
 	cptr->serv->snum = find_server_num(cptr->name);
-#endif
 	cptr->serv->stok = 1;
 	cptr->flags |= FLAGS_CBURST;
 	(void) add_to_server_hash_table(cptr->serv, cptr);
@@ -730,7 +797,7 @@ Reg	aClient	*cptr;
 		    acptr == cptr || IsMe(acptr))		    
 			continue;
 		if ((aconf = acptr->serv->nline) &&
-		    !matches(my_name_for_link(ME, aconf->port), cptr->name))
+		    !match(my_name_for_link(ME, aconf->port), cptr->name))
 			continue;
 		stok = (acptr->serv->version != SV_OLD) ? cptr->serv->tok : "";
 		if (split)
@@ -767,7 +834,7 @@ Reg	aClient	*cptr;
 		/* acptr->from == acptr for acptr == cptr */
 		if ((acptr->from == cptr) || !IsServer(acptr))
 			continue;
-		if (*mlname == '*' && matches(mlname, acptr->name) == 0)
+		if (*mlname == '*' && match(mlname, acptr->name) == 0)
 			continue;
 		split = (MyConnect(acptr) &&
 			 mycmp(acptr->name, acptr->sockhost));
@@ -795,10 +862,16 @@ Reg	aClient	*cptr;
 			** These are only true when *BOTH* NICK and USER have
 			** been received. -avalon
 			*/
+			if (acptr->user->servp->userlist == NULL)
+				sendto_flag(SCH_ERROR,
+			    "ERROR: USER:%s without SERVER:%s(%d) (to %s)",
+					    acptr->name, acptr->user->server,
+					    acptr->user->servp->tok,
+					    cptr->name);
 			if (i != SV_OLD)
 			    {
 				if (*mlname == '*' &&
-				    matches(mlname, acptr->user->server) == 0)
+				    match(mlname, acptr->user->server) == 0)
 					stok = me.serv->tok;
 				else
 					stok = acptr->user->servp->tok;
@@ -822,16 +895,20 @@ Reg	aClient	*cptr;
 			send_user_joins(cptr, acptr);
 		    }
 		else if (IsService(acptr) &&
-			 (matches(acptr->service->dist, acptr->name) == 0
+			 (match(acptr->service->dist, cptr->name) == 0
 			  && cptr->serv->version != SV_OLD))
 		    {
-			stok = i ? acptr->service->servp->tok :
-				   acptr->service->server;
+			if (*mlname == '*' &&
+			    match(mlname, acptr->service->server) == 0)
+				stok = me.serv->tok;
+			else
+				stok = acptr->service->servp->tok;
 			sendto_one(cptr, "SERVICE %s %s %s %d %d :%s",
 				   acptr->name, stok, acptr->service->dist,
 				   acptr->service->type, acptr->hopcount + 1,
 				   acptr->info);
 		    }
+		/* the previous if does NOT catch all services.. ! */
 	    }
 	/*
 	** Last, pass all channels modes
@@ -844,6 +921,18 @@ Reg	aClient	*cptr;
 				send_channel_modes(cptr, chptr);
 	}
 	cptr->flags &= ~FLAGS_CBURST;
+#ifdef	ZIP_LINKS
+ 	/*
+ 	** some stats about the connect burst,
+ 	** they are slightly incorrect because of cptr->zip->outbuf.
+ 	*/
+ 	if ((cptr->flags & FLAGS_ZIP) && cptr->zip->out->total_in)
+	  sendto_flag(SCH_NOTICE,
+		      "Connect burst to %s: %lu, compressed: %lu (%3.1f%%)",
+		      get_client_name(cptr, TRUE),
+		      cptr->zip->out->total_in,cptr->zip->out->total_out,
+ 	    (float) 100*cptr->zip->out->total_out/cptr->zip->out->total_in);
+#endif
 	return 0;
 }
 
@@ -861,7 +950,7 @@ char	*parv[];
 		return exit_client(cptr, sptr, &me, "Already registered");
 
 	if (parc < 3)
-		return 0;
+		return 1;
 
 	name = parv[1];
 
@@ -871,7 +960,8 @@ char	*parv[];
 		    bcmp((char *)&acptr->ip, (char *)&cptr->ip,
 			 sizeof(acptr->ip)) || mycmp(acptr->name, name))
 			continue;
-		if (!(aconf = find_conf_name(name, CONF_CONNECT_SERVER)) ||
+		if (!(aconf = find_conf_name(name, CONF_CONNECT_SERVER|
+					     CONF_ZCONNECT_SERVER)) ||
 		    atoi(parv[2]) != acptr->receiveM)
 			break;
 		attach_confs(acptr, name, CONF_SERVER_MASK);
@@ -917,10 +1007,10 @@ char	*parv[];
 			   ME, RPL_INFO, parv[0],
 			   myctime(me.firsttime));
 		sendto_one(sptr, rpl_str(RPL_ENDOFINFO, parv[0]));
-		return 3;
+		return 5;
 	    }
 	else
-		return 5;
+		return 10;
 }
 
 /*
@@ -964,7 +1054,7 @@ char	*parv[];
 
 	sendto_one(sptr, rpl_str(RPL_ENDOFLINKS, parv[0]),
 		   BadPtr(mask) ? "*" : mask);
-	return 1;
+	return 2;
 }
 
 /*
@@ -1071,8 +1161,8 @@ char	*parv[];
 #endif /* ENABLE_SUMMON */
 	    }
 	else
-		return 2;
-	return 1;
+		return 3;
+	return 2;
 }
 
 
@@ -1098,11 +1188,13 @@ char	*parv[];
 **	      it--not reversed as in ircd.conf!
 */
 
-static int report_array[12][3] = {
+static int report_array[16][3] = {
+		{ CONF_ZCONNECT_SERVER,	  RPL_STATSCLINE, 'c'},
 		{ CONF_CONNECT_SERVER,	  RPL_STATSCLINE, 'C'},
 		{ CONF_NOCONNECT_SERVER,  RPL_STATSNLINE, 'N'},
 		{ CONF_CLIENT,		  RPL_STATSILINE, 'I'},
 		{ CONF_RCLIENT,		  RPL_STATSILINE, 'i'},
+		{ CONF_OTHERKILL,	  RPL_STATSKLINE, 'k'},
 		{ CONF_KILL,		  RPL_STATSKLINE, 'K'},
 		{ CONF_QUARANTINED_SERVER,RPL_STATSQLINE, 'Q'},
 		{ CONF_LEAF,		  RPL_STATSLLINE, 'L'},
@@ -1110,6 +1202,8 @@ static int report_array[12][3] = {
 		{ CONF_HUB,		  RPL_STATSHLINE, 'H'},
 		{ CONF_LOCOP,		  RPL_STATSOLINE, 'o'},
 		{ CONF_SERVICE,		  RPL_STATSSLINE, 'S'},
+		{ CONF_VER,		  RPL_STATSVLINE, 'V'},
+		{ CONF_BOUNCE,		  RPL_STATSBLINE, 'B'},
 		{ 0, 0, 0}
 	};
 
@@ -1133,18 +1227,22 @@ int	mask;
 				continue;
 			c = (char)*(p+2);
 			host = BadPtr(tmp->host) ? null : tmp->host;
-			pass = BadPtr(tmp->passwd) ? null : tmp->passwd;
+			pass = BadPtr(tmp->passwd) ? NULL : tmp->passwd;
 			name = BadPtr(tmp->name) ? null : tmp->name;
 			port = (int)tmp->port;
 			/*
-			 * On K line the passwd contents can be
+			 * On K/V lines the passwd contents can be
 			 * displayed on STATS reply. 	-Vesa
 			 */
-			if (tmp->status == CONF_KILL)
-				sendto_one(sptr, rpl_str(p[1], to),c,host,pass,
+			if (tmp->status == CONF_KILL
+			    || tmp->status == CONF_OTHERKILL
+			    || tmp->status == CONF_VER)
+				sendto_one(sptr, rpl_str(p[1], to), c, host,
+					   (pass) ? pass : null,
 					   name, port, get_conf_class(tmp));
 			else
 				sendto_one(sptr, rpl_str(p[1], to), c, host,
+					   (pass) ? "*" : null,
 					   name, port, get_conf_class(tmp));
 		    }
 	return;
@@ -1203,7 +1301,7 @@ char	*parv[];
 		name = parv[2];
 		if (!mycmp(name, ME))
 			doall = 2;
-		else if (matches(name, ME) == 0)
+		else if (match(name, ME) == 0)
 			doall = 1;
 		if (index(name, '*') || index(name, '?'))
 			wilds = 1;
@@ -1241,11 +1339,11 @@ char	*parv[];
 			    !(MyConnect(sptr) && IsAnOper(sptr)) &&
 			    acptr != sptr)
 				continue;
-			if (!doall && wilds && matches(name, acptr->name))
+			if (!doall && wilds && match(name, acptr->name))
 				continue;
 			if (!(doall || wilds) &&
 			    ((!cm && mycmp(name, acptr->name)) ||
-			     (cm && matches(cm, acptr->name))))
+			     (cm && match(cm, acptr->name))))
 				continue;
 			sendto_one(cptr, Lformat, ME,
 				   RPL_STATSLINKINFO, parv[0],
@@ -1256,8 +1354,12 @@ char	*parv[];
 				   timeofday - acptr->firsttime);
 		    }
 		break;
-	case 'C' : case 'c' : /* C and N conf lines */
+	case 'B' : case 'b' : /* B conf lines */
+		report_configured_links(cptr, parv[0], CONF_BOUNCE);
+		break;
+	case 'c' : case 'C' : /* C and N conf lines */
 		report_configured_links(cptr, parv[0], CONF_CONNECT_SERVER|
+					CONF_ZCONNECT_SERVER|
 					CONF_NOCONNECT_SERVER);
 		break;
 	case 'd' : case 'D' : /* defines */
@@ -1271,7 +1373,8 @@ char	*parv[];
 					CONF_CLIENT|CONF_RCLIENT);
 		break;
 	case 'K' : case 'k' : /* K lines */
-		report_configured_links(cptr, parv[0], CONF_KILL);
+		report_configured_links(cptr, parv[0],
+					(CONF_KILL|CONF_OTHERKILL));
 		break;
 	case 'M' : case 'm' : /* commands use/stats */
 		for (mptr = msgtab; mptr->cmd; mptr++)
@@ -1309,6 +1412,9 @@ char	*parv[];
 			   now/86400, (now/3600)%24, (now/60)%60, now%60);
 		break;
 	    }
+	case 'V' : case 'v' : /* V conf lines */
+		report_configured_links(cptr, parv[0], CONF_VER);
+		break;
 	case 'X' : case 'x' : /* lists */
 #ifdef	DEBUGMODE
 		send_listinfo(cptr, parv[0]);
@@ -1377,8 +1483,8 @@ char	*parv[];
 #endif
 	    }
 	else
-		return 2;
-	return 1;
+		return 3;
+	return 2;
 }
 
 /*
@@ -1414,7 +1520,7 @@ char	*parv[];
 	else
 		sendto_flag(SCH_ERROR, "from %s via %s -- %s",
 			   sptr->name, get_client_name(cptr,FALSE), para);
-	return 1;
+	return 2;
     }
 
 /*
@@ -1458,7 +1564,7 @@ char	*parv[];
 	if (parc > 2)
 		if(hunt_server(cptr, sptr, ":%s LUSERS %s :%s", 2, parc, parv)
 				!= HUNTED_ISME)
-			return 2;
+			return 3;
 
 	if (parc == 1)
 	    {
@@ -1477,7 +1583,7 @@ char	*parv[];
 		sendto_one(sptr, rpl_str(RPL_LUSERME, parv[0]),
 			   istat.is_myclnt, istat.is_myservice,
 			   istat.is_myserv);
-		return 1;
+		return 2;
 	    }
 	(void)collapse(parv[1]);
 	for (acptr = client; acptr; acptr = acptr->next)
@@ -1556,7 +1662,7 @@ char	*parv[];
 			   count_channels(sptr));
 	sendto_one(sptr, rpl_str(RPL_LUSERME, parv[0]), m_client, m_service,
 		   m_server);
-	return 1;
+	return 2;
     }
 
   
@@ -1588,7 +1694,7 @@ char	*parv[];
 
 	if (hunt_server(cptr,sptr,":%s CONNECT %s %s :%s",
 		       3,parc,parv) != HUNTED_ISME)
-		return 0;
+		return 1;
 
 	if (parc < 3 || *parv[1] == '\0')
 	    {
@@ -1607,15 +1713,17 @@ char	*parv[];
 	    }
 
 	for (aconf = conf; aconf; aconf = aconf->next)
-		if (aconf->status == CONF_CONNECT_SERVER &&
-		    matches(parv[1], aconf->name) == 0)
+		if ((aconf->status == CONF_CONNECT_SERVER ||
+		     aconf->status == CONF_ZCONNECT_SERVER) &&
+		    match(parv[1], aconf->name) == 0)
 		  break;
 	/* Checked first servernames, then try hostnames. */
 	if (!aconf)
 		for (aconf = conf; aconf; aconf = aconf->next)
-			if (aconf->status == CONF_CONNECT_SERVER &&
-			    (matches(parv[1], aconf->host) == 0 ||
-			     matches(parv[1], index(aconf->host, '@')+1) == 0))
+			if ((aconf->status == CONF_CONNECT_SERVER ||
+			     aconf->status == CONF_ZCONNECT_SERVER) &&
+			    (match(parv[1], aconf->host) == 0 ||
+			     match(parv[1], index(aconf->host, '@')+1) == 0))
 		  		break;
 
 	if (!aconf)
@@ -1709,7 +1817,7 @@ char	*parv[];
 	    {
 		sendto_one(sptr, err_str(ERR_NEEDMOREPARAMS, parv[0]),
 			   "WALLOPS");
-		return 0;
+		return 1;
 	    }
 
 	if (!IsServer(sptr))
@@ -1738,14 +1846,11 @@ int	m_time(cptr, sptr, parc, parv)
 aClient *cptr, *sptr;
 int	parc;
 char	*parv[];
-    {
+{
 	if (hunt_server(cptr,sptr,":%s TIME :%s",1,parc,parv) == HUNTED_ISME)
-	    {
 		sendto_one(sptr, rpl_str(RPL_TIME, parv[0]), ME, date((long)0));
-		return 1;
-	    }
-	return 1;
-    }
+	return 2;
+}
 
 
 /*
@@ -1762,8 +1867,9 @@ char	*parv[];
 
 	if (IsRegistered(cptr) &&	/* only local query for unregistered */
 	    hunt_server(cptr,sptr,":%s ADMIN :%s",1,parc,parv) != HUNTED_ISME)
-		return 2;
-	if ((aconf = find_admin()))
+		return 3;
+	if ((aconf = find_admin()) && aconf->host && aconf->passwd
+	    && aconf->name)
 	    {
 		sendto_one(sptr, rpl_str(RPL_ADMINME, parv[0]), ME);
 		sendto_one(sptr, rpl_str(RPL_ADMINLOC1, parv[0]), aconf->host);
@@ -1774,7 +1880,7 @@ char	*parv[];
 	    }
 	else
 		sendto_one(sptr, err_str(ERR_NOADMININFO, parv[0]), ME);
-	return 1;
+	return 2;
     }
 
 #if defined(OPER_REHASH) || defined(LOCOP_REHASH)
@@ -1844,8 +1950,13 @@ char	*parv[];
 
 		ac2ptr = next_client(client, parv[1]);
 		sendto_one(sptr, rpl_str(RPL_TRACELINK, parv[0]),
-			   version, debugmode, tname, ac2ptr->from->name);
-		return 2;
+			   version, debugmode, tname, ac2ptr->from->name,
+			   ac2ptr->from->serv->version,
+			   (ac2ptr->from->flags & FLAGS_ZIP) ? "z" : "",
+			   timeofday - ac2ptr->from->firsttime,
+			   (int)DBufLength(&ac2ptr->from->sendQ),
+			   (int)DBufLength(&sptr->from->sendQ));
+		return 5;
 	    }
 	case HUNTED_ISME:
 		break;
@@ -1853,7 +1964,7 @@ char	*parv[];
 		return 1;
 	}
 
-	doall = (parv[1] && (parc > 1)) ? !matches(tname, ME): TRUE;
+	doall = (parv[1] && (parc > 1)) ? !match(tname, ME): TRUE;
 	wilds = !parv[1] || index(tname, '*') || index(tname, '?');
 	dow = wilds || doall;
 
@@ -1886,7 +1997,7 @@ char	*parv[];
 		    !(MyConnect(sptr) && IsAnOper(sptr)) &&
 		    !IsAnOper(acptr) && (acptr != sptr))
 			continue;
-		if (!doall && wilds && matches(tname, acptr->name))
+		if (!doall && wilds && match(tname, acptr->name))
 			continue;
 		if (!dow && mycmp(tname, acptr->name))
 			continue;
@@ -1949,15 +2060,16 @@ char	*parv[];
 					   link_u[i], name, acptr->serv->by,
 					   acptr->serv->user->username,
 					   acptr->serv->user->host,
-					   acptr->serv->version);
+					   acptr->serv->version,
+					   (acptr->flags & FLAGS_ZIP) ?"z":"");
 			else
 				sendto_one(sptr, rpl_str(RPL_TRACESERVER,
 					   parv[0]), class, link_s[i],
 					   link_u[i], name,
 					   *(acptr->serv->by) ?
 					   acptr->serv->by : "*", "*", ME,
-					   acptr->serv->version);
-			break;
+					   acptr->serv->version,
+					   (acptr->flags & FLAGS_ZIP) ?"z":"");			break;
 		case STAT_RECONNECT:
 			sendto_one(sptr, rpl_str(RPL_TRACERECONNECT, parv[0]),
 				   class, name);
@@ -1989,7 +2101,7 @@ char	*parv[];
 				   Class(cltmp), Links(cltmp));
 	sendto_one(sptr, rpl_str(RPL_TRACEEND, parv[0]), tname, version,
 		   debugmode);
-	return 1;
+	return 2;
     }
 
 /*
@@ -2002,14 +2114,36 @@ aClient *cptr, *sptr;
 int	parc;
 char	*parv[];
 {
+#ifdef	CACHED_MOTD
+	register aMotd *temp;
+	struct tm *tm;
+#else
 	int	fd;
 	char	line[80];
 	Reg	char	 *tmp;
 	struct	stat	sb;
 	struct	tm	*tm;
+#endif
 
 	if (hunt_server(cptr, sptr, ":%s MOTD :%s", 1,parc,parv)!=HUNTED_ISME)
+		return 5;
+#ifdef CACHED_MOTD
+	tm = &motd_tm;
+	if (motd == NULL)
+	    {
+		sendto_one(sptr, err_str(ERR_NOMOTD, parv[0]));
 		return 1;
+	    }
+	sendto_one(sptr, rpl_str(RPL_MOTDSTART, parv[0]), ME);
+	sendto_one(sptr, ":%s %d %s :- %d/%d/%d %d:%02d", ME, RPL_MOTD,
+		   parv[0], tm->tm_mday, tm->tm_mon + 1, 1900 + tm->tm_year,
+		   tm->tm_hour, tm->tm_min);
+	temp = motd;
+	for(temp=motd;temp != NULL;temp = temp->next)
+		sendto_one(sptr, rpl_str(RPL_MOTD, parv[0]), temp->line);
+	sendto_one(sptr, rpl_str(RPL_ENDOFMOTD, parv[0]));
+	return 2;
+#else
 	/*
 	 * stop NFS hangs...most systems should be able to open a file in
 	 * 3 seconds. -avalon (curtesy of wumpus)
@@ -2040,8 +2174,9 @@ char	*parv[];
 	(void)dgets(-1, NULL, 0); /* make sure buffer is at empty pos */
 	sendto_one(sptr, rpl_str(RPL_ENDOFMOTD, parv[0]));
 	(void)close(fd);
-	return 1;
-    }
+	return 2;
+#endif	/* CACHED_MOTD */
+}
 
 /*
 ** m_close - added by Darren Reed Jul 13 1992.
@@ -2068,7 +2203,7 @@ char	*parv[];
 		closed++;
 	    }
 	sendto_one(sptr, rpl_str(RPL_CLOSEEND, parv[0]), closed);
-	return 0;
+	return 1;
 }
 
 #if defined(OPER_DIE) || defined(LOCOP_DIE)
@@ -2079,7 +2214,9 @@ char	*parv[];
 {
 	Reg	aClient	*acptr;
 	Reg	int	i;
+	char	killer[HOSTLEN * 2 + USERLEN + 5];
 
+	strcpy(killer, get_client_name(sptr, TRUE));
 	for (i = 0; i <= highest_fd; i++)
 	    {
 		if (!(acptr = local[i]))
@@ -2088,30 +2225,25 @@ char	*parv[];
 		    {
 			sendto_one(acptr,
 				   ":%s NOTICE %s :Server Terminating. %s",
-				   ME, acptr->name,
-				   get_client_name(sptr, TRUE));
+				   ME, acptr->name, killer);
 			acptr->exitc = EXITC_DIE;
 			(void)exit_client(acptr, acptr, &me, "Server died");
 		    }
 		else if (IsServer(acptr))
 			sendto_one(acptr, ":%s ERROR :Terminated by %s",
-				   ME, get_client_name(sptr, TRUE));
+				   ME, killer);
 	    }
 	(void)s_die();
 	return 0;
 }
 #endif
 
-#ifdef KRYS
 /*
 ** storing server names in User structures is a real waste,
 ** the following functions change it to only store a pointer.
 ** A better way might be to store in Server structure and use servp. -krys
 */
 
-/* The following has not been tested much, it seems to work, but
- * if you #define it and something bad happens, don't blame me. :-)
- */
 static char	**server_name = NULL;
 static int	server_max = 0, server_num = 0;
 
@@ -2157,10 +2289,12 @@ char *sname;
 	if (i == server_max)
 	  {
 	    /* server_name[] array is full, let's make it bigger! */
-	    server_name = (char **) MyRealloc((char *)server_name,
+	    if (server_name)
+		    server_name = (char **) MyRealloc((char *)server_name,
 					      sizeof(char *)*(server_max+=50));
+	    else
+		    server_name = (char **) MyMalloc(sizeof(char *)*(server_max=50));
 	  }
 	server_name[server_num] = mystrdup(sname);
 	return server_num++;
 }
-#endif
