@@ -22,8 +22,7 @@
  */
 
 #ifndef lint
-static  char sccsid[] = "%W% %G% (C) 1988 University of Oulu, \
-Computing Center and Jarkko Oikarinen";
+static  char rcsid[] = "@(#)$Id: s_user.c,v 1.15 1997/07/15 20:58:00 kalt Exp $";
 #endif
 
 #include <sys/types.h>	/* HPUX requires sys/types.h for utmp.h */
@@ -125,7 +124,7 @@ Reg	char	*ch;	/* search string (may include wilds) */
 	    {
 		if (IsService(next))
 			continue;
-		if (!matches(ch,next->name) || !matches(next->name,ch))
+		if (!match(ch,next->name) || !match(next->name,ch))
 			break;
 	    }
 	return next;
@@ -163,8 +162,8 @@ int	server, parc;
 	** Assume it's me, if no server
 	*/
 	if (parc <= server || BadPtr(parv[server]) ||
-	    matches(ME, parv[server]) == 0 ||
-	    matches(parv[server], ME) == 0)
+	    match(ME, parv[server]) == 0 ||
+	    match(parv[server], ME) == 0)
 		return (HUNTED_ISME);
 	/*
 	** These are to pickup matches that would cause the following
@@ -174,10 +173,12 @@ int	server, parc;
 	if ((acptr = find_client(parv[server], NULL)))
 		if (acptr->from == sptr->from && !MyConnect(acptr))
 			acptr = NULL;
+	/* Match *.masked.servers */
 	if (!acptr && (acptr = find_server(parv[server], NULL)))
 		if (acptr->from == sptr->from && !MyConnect(acptr))
 			acptr = NULL;
-	if (!acptr && (acptr = find_nickserv(parv[server], NULL)))
+	/* Remote services@servers */
+	if (!acptr && (acptr = find_service(parv[server], NULL)))
 		if (acptr->from == sptr->from && !MyConnect(acptr))
 			acptr = NULL;
 	if (!acptr)
@@ -199,7 +200,7 @@ int	server, parc;
 	    {
 		if (IsMe(acptr) || MyClient(acptr))
 			return HUNTED_ISME;
-		if (matches(acptr->name, parv[server]))
+		if (match(acptr->name, parv[server]))
 			parv[server] = acptr->name;
 		/*
 	        ** Stop services to talk to 'OLD' servers and
@@ -208,7 +209,7 @@ int	server, parc;
 		if (IsService(sptr)
 		    && (IsServer(acptr->from)
 			&& (acptr->from->serv->version == SV_OLD
-			    || matches(sptr->service->dist,acptr->name) != 0)))
+			    || match(sptr->service->dist,acptr->name) != 0)))
 		    {
 			sendto_one(sptr, err_str(ERR_NOSUCHSERVER, parv[0]), 
 				   parv[server]);
@@ -348,6 +349,8 @@ char	*nick, *username;
 
 	if (MyConnect(sptr))
 	    {
+		char *reason = NULL;
+
 		if ((i = check_client(sptr)))
 		    {
 			ircstp->is_ref++;
@@ -366,17 +369,15 @@ char	*nick, *username;
 			       " No more " : " No Auth ",
 			       (IsUnixSocket(sptr)) ? me.sockhost :
 			       ((sptr->hostp) ? sptr->hostp->h_name :
-				sptr->sockhost), sptr->username, sptr->exitc);
+				sptr->sockhost), sptr->auth, sptr->exitc);
 #endif		    
 #ifdef FNAME_CONNLOG
-			sendto_flog(myctime(sptr->firsttime), 
-				    (i == -5) ? " u@h max " :
+			sendto_flog(sptr, (i == -5) ? " u@h max " :
 				    (i == -4) ? " IP  max " : (i == -3) ? 
 				    " No more " : " No Auth ", 0, "<none>",
 				    (IsUnixSocket(sptr)) ? me.sockhost :
 				    ((sptr->hostp) ? sptr->hostp->h_name :
-				    sptr->sockhost), sptr->username,
-				    &sptr->exitc);
+				    sptr->sockhost));
 #endif
 			return exit_client(cptr, sptr, &me,
 				(i == -5) ?
@@ -392,6 +393,7 @@ char	*nick, *username;
 		else
 			strncpyzt(user->host, sptr->sockhost, HOSTLEN+1);
 
+#ifndef	NO_PREFIX
 		/*
 		** ident is fun.. ahem
 		** prefixes used:
@@ -425,6 +427,7 @@ char	*nick, *username;
 			strncpyzt(buf2, sptr->username, USERLEN+1);
 		else /* No ident, or unusable ident string */
 		     /* because username may point to user->username */
+#endif
 			strncpyzt(buf2, username, USERLEN+1);
 
 		if (prefix)
@@ -447,40 +450,43 @@ char	*nick, *username;
 		/*
 		 * following block for the benefit of time-dependent K:-lines
 		 */
-		if (find_kill(sptr, 1))
+		if (find_kill(sptr, 1, &reason))
 		    {
+			/*char buf[100];*/
+
 			sendto_flag(SCH_LOCAL, "K-lined %s@%s.",
-				    sptr->user->username, sptr->sockhost);
+				    user->username, sptr->sockhost);
 			ircstp->is_ref++;
 			sptr->exitc = EXITC_REF;
 #if defined(USE_SYSLOG) && defined(SYSLOG_CONN)
 			syslog(LOG_NOTICE, "%s ( K lined ): %s@%s [%s] %c\n",
-			       myctime(sptr->firsttime), sptr->user->username,
-			       sptr->user->host, sptr->username, '-');
+			       myctime(sptr->firsttime), user->username,
+			       user->host, sptr->auth, '-');
 #endif		    
 #ifdef FNAME_CONNLOG
-			sendto_flog(myctime(sptr->firsttime), " K lined ", 0,
-				    sptr->user->username, sptr->user->host,
-				    sptr->username, "-");
+			sendto_flog(sptr, " K lined ", 0, user->username,
+				    user->host);
 #endif
-			return exit_client(cptr, sptr, &me, "K-lined");
+			if (reason)
+				sprintf(buf, "K-lined: %.80s", reason);
+			return exit_client(cptr, sptr, &me, (reason) ? buf :
+					   "K-lined");
 		    }
 #ifdef R_LINES
 		if (find_restrict(sptr))
 		    {
 			sendto_flag(SCH_LOCAL, "R-lined %s@%s.",
-				    sptr->user->username, sptr->sockhost);
+				    user->username, sptr->sockhost);
 			ircstp->is_ref++;
 			sptr->exitc = EXITC_REF;
 # if defined(USE_SYSLOG) && defined(SYSLOG_CONN)
 			syslog(LOG_NOTICE, "%s ( R lined ): %s@%s [%s] %c\n",
-			       myctime(sptr->firsttime), sptr->user->username,
-			       sptr->user->host, sptr->username, '-');
+			       myctime(sptr->firsttime), user->username,
+			       user->host, sptr->username, '-');
 # endif		    
 # ifdef FNAME_CONNLOG
-			sendto_flog(myctime(sptr->firsttime), " R lined ", 0,
-				    sptr->user->username, sptr->user->host,
-				    sptr->username, "-");
+			sendto_flog(sptr, " R lined ", 0, user->username,
+				    user->host);
 # endif
 			return exit_client(cptr, sptr, &me , "R-lined");
 		    }
@@ -496,8 +502,9 @@ char	*nick, *username;
 	SetClient(sptr);
 	if (MyConnect(sptr))
 	    {
+		sprintf(buf, "%s!%s@%s", nick, user->username, user->host);
 		sptr->exitc = EXITC_REG;
-		sendto_one(sptr, rpl_str(RPL_WELCOME, nick), nick);
+		sendto_one(sptr, rpl_str(RPL_WELCOME, nick), buf);
 		/* This is a duplicate of the NOTICE but see below...*/
 		sendto_one(sptr, rpl_str(RPL_YOURHOST, nick),
 			   get_client_name(&me, FALSE), version);
@@ -529,7 +536,7 @@ char	*nick, *username;
 			continue;
 		if (acptr->serv->version != SV_OLD)
 			if ((aconf = acptr->serv->nline) &&
-			    !matches(my_name_for_link(ME, aconf->port),
+			    !match(my_name_for_link(ME, aconf->port),
 				     user->server))
 				sendto_one(acptr, "NICK %s %d %s %s %s %s :%s",
 					   nick, sptr->hopcount+1, 
@@ -575,8 +582,9 @@ char	*nick, *username;
 		istat.is_myclnt++;
 	    }
 #ifdef	USE_SERVICES
-	check_services_butone(SERVICE_WANT_NICK, user->server, sptr,
-			      "NICK %s :%d", nick, sptr->hopcount);
+#if 0
+	check_services_butone(SERVICE_WANT_NICK, user->server, NULL,
+			      "NICK %s :%d", nick, sptr->hopcount+1);
 	check_services_butone(SERVICE_WANT_USER, user->server, sptr,
 			      ":%s USER %s %s %s :%s", nick, user->username, 
 			      user->host, user->server, sptr->info);
@@ -584,6 +592,10 @@ char	*nick, *username;
 		send_umode(NULL, sptr, 0, ALL_UMODES, buf);
 	check_services_butone(SERVICE_WANT_UMODE, user->server, sptr,
 			      ":%s MODE %s :%s", nick, nick, buf);
+#endif
+	if (MyConnect(sptr))	/* all modes about local users */
+		send_umode(NULL, sptr, 0, ALL_UMODES, buf);
+	check_services_num(sptr, buf);
 #endif
 #ifdef NPATH
 	note_signon(sptr);
@@ -674,7 +686,7 @@ char	*parv[];
 				return exit_client(cptr,sptr,&me,"BadNick");
 			    }
 		    }
-		return 1;
+		return 2;
 	    }
 
 	/*
@@ -690,7 +702,7 @@ char	*parv[];
 		    {
 			sendto_one(sptr, err_str(ERR_NICKNAMEINUSE, parv[0]),
 				   nick);
-			return 1; /* NICK message ignored */
+			return 2; /* NICK message ignored */
 		    }
 	/*
 	** acptr already has result from previous find_server()
@@ -704,8 +716,12 @@ char	*parv[];
 		** there is no danger of the server being disconnected.
 		** Ultimate way to jupiter a nick ? >;-). -avalon
 		*/
-		sendto_flag(SCH_KILL, "Nick collision on %s (%s@%s)%s <- %s",
-			    sptr->name, user, host, acptr->from->name,
+		sendto_flag(SCH_KILL,
+			    "Nick collision on %s (%s@%s)%s <- (%s@%s)%s",
+			    sptr->name,
+			    (acptr->user) ? acptr->user->username : "???",
+			    (acptr->user) ? acptr->user->host : "???",
+			    acptr->from->name, user, host,
 			    get_client_name(cptr, FALSE));
 		ircstp->is_kill++;
 		sendto_one(cptr, ":%s KILL %s :%s (%s <- %s)",
@@ -747,7 +763,7 @@ char	*parv[];
 			** especially since servers prior to this
 			** version would treat it as nick collision.
 			*/
-			return 1; /* NICK Message ignored */
+			return 2; /* NICK Message ignored */
 	/*
 	** Note: From this point forward it can be assumed that
 	** acptr != sptr (point to different client structures).
@@ -775,7 +791,7 @@ char	*parv[];
 		sendto_one(sptr, err_str((delayed) ? ERR_UNAVAILRESOURCE
 						   : ERR_NICKNAMEINUSE,
 					 parv[0]), nick);
-		return 1; /* NICK message ignored */
+		return 2; /* NICK message ignored */
 	    }
 	/*
 	** NICK was coming from a server connection. Means that the same
@@ -796,31 +812,29 @@ char	*parv[];
 	*/
 	if (sptr == cptr)
 	    {
-		sendto_flag(SCH_KILL, "Nick collision on %s (%s@%s)%s <- %s",
-			    acptr->name, user, host, acptr->from->name,
-			    get_client_name(cptr, FALSE));
+		sendto_flag(SCH_KILL,
+			    "Nick collision on %s (%s@%s)%s <- (%s@%s)%s",
+			    acptr->name,
+			    (acptr->user) ? acptr->user->username : "???",
+			    (acptr->user) ? acptr->user->host : "???",
+			    acptr->from->name,
+			    user, host, get_client_name(cptr, FALSE));
 		/*
 		** A new NICK being introduced by a neighbouring
 		** server (e.g. message type "NICK new" received)
 		*/
 		ircstp->is_kill++;
-		sendto_serv_butone(cptr, ":%s KILL %s :%s ((%s@%s)%s <- %s)",
-				   ME, acptr->name, ME, user, host,
-				   acptr->from->name,
+		sendto_serv_butone(NULL, 
+				   ":%s KILL %s :%s ((%s@%s)%s <- (%s@%s)%s)",
+				   ME, acptr->name, ME,
+				   (acptr->user) ? acptr->user->username:"???",
+				   (acptr->user) ? acptr->user->host : "???",
+				   acptr->from->name, user, host,
 				   /* NOTE: Cannot use get_client_name twice
 				   ** here, it returns static string pointer:
 				   ** the other info would be lost
 				   */
 				   get_client_name(cptr, FALSE));
-		sendto_one(cptr, ":%s KILL %s :%s (%s <- (%s@%s)%s)",
-			    ME, acptr->name, ME, acptr->from->name,
-			    acptr->username,
-			    (acptr->user) ? acptr->user->host : "???",
-			    /* NOTE: Cannot use get_client_name twice
-			    ** here, it returns static string pointer:
-			    ** the other info would be lost
-			    */
-			    get_client_name(cptr, FALSE));
 		acptr->flags |= FLAGS_KILLED;
 		return exit_client(NULL, acptr, &me, "Nick collision");
 	    }
@@ -879,12 +893,12 @@ nickkilldone:
 		if (MyConnect(sptr))
 		{
 			if (!IsPerson(sptr))    /* Unregistered client */
-				return 1;       /* Ignore new NICKs */
+				return 2;       /* Ignore new NICKs */
 			if (IsRestricted(sptr))
 			    {
 				sendto_one(sptr,
 					   err_str(ERR_RESTRICTED, nick));
-				return 1;
+				return 2;
 			    }
 		}
 		/*
@@ -931,7 +945,7 @@ nickkilldone:
 	**  Finally set new nick name.
 	*/
 	(void)add_to_client_hash_table(nick, sptr);
-	return 2;
+	return 3;
 }
 
 /*
@@ -950,13 +964,13 @@ nickkilldone:
 static	int	m_message(cptr, sptr, parc, parv, notice)
 aClient *cptr, *sptr;
 char	*parv[];
-int	notice;
+int	parc, notice;
 {
 	Reg	aClient	*acptr;
 	Reg	char	*s;
 	aChannel *chptr;
-	char	*nick, *server, *p, *cmd, *host;
-	int	count = 0, nickcnt = 0;
+	char	*nick, *server, *p, *cmd, *user, *host;
+	int	count = 0, penalty = 0;
 
 	cmd = notice ? MSG_NOTICE : MSG_PRIVATE;
 
@@ -975,22 +989,19 @@ int	notice;
 	if (MyConnect(sptr))
 		parv[1] = canonize(parv[1]);
 	for (p = NULL, nick = strtoken(&p, parv[1], ","); nick;
-	     nick = strtoken(&p, NULL, ","), nickcnt++)
+	     nick = strtoken(&p, NULL, ","), penalty++)
 	    {
-
 		/*
-		** restrict destination list to MAXMSGDESTS recipients to
-		** solve SPAM problem --Yegg
-		*/
-#define		MAXMSGDESTS 5
-		if (nickcnt >= MAXMSGDESTS) {
+		** restrict destination list to MAXPENALTY/2 recipients to
+		** solve SPAM problem --Yegg 
+		*/ 
+		if (2*penalty >= MAXPENALTY) {
 		    if (!notice)
-			sendto_one(sptr, err_str(
-				   ERR_TOOMANYDESTS,
-				   parv[0]), nick);
-		    continue;
-		}
-
+			    sendto_one(sptr, err_str(ERR_TOOMANYTARGETS,
+						     parv[0]),
+				       "Too many", nick); 
+		    continue;      
+		}   
 		/*
 		** nickname addressed?
 		*/
@@ -1043,7 +1054,9 @@ int	notice;
 					   parv[0]), nick);
 				continue;
 			    }
-			if (matches(nick + 1, sptr->user->server))
+			if (match(nick + 1, sptr->user->server) &&
+			    (*nick == '$' ||
+			     match(nick + 1, sptr->user->host)))
 			    {
 				sendto_one(sptr, err_str(ERR_BADMASK,
 					   parv[0]), nick);
@@ -1057,7 +1070,29 @@ int	notice;
 					    cmd, nick, parv[2]);
 			continue;
 		    }
-	
+		
+		/*
+		** nick!user@host addressed?
+		*/
+		if ((user = (char *)index(nick, '!')) &&
+		    (host = (char *)index(nick, '@')))
+		    {
+			*user = '\0';
+			*host = '\0';
+			if ((acptr = find_person(nick, NULL)) &&
+			    !mycmp(user+1, acptr->user->username) &&
+			    !mycmp(host+1, acptr->user->host))
+			    {
+				sendto_prefix_one(acptr, sptr, ":%s %s %s :%s",
+						  parv[0], cmd, nick, parv[2]);
+				*user = '!';
+				*host = '@';
+				continue;
+			    }
+			*user = '!';
+			*host = '@';
+		    }
+
 		/*
 		** user[%host]@server addressed?
 		*/
@@ -1098,7 +1133,7 @@ int	notice;
 				else if (!notice)
 					sendto_one(sptr, err_str(
 						   ERR_TOOMANYTARGETS,
-						   parv[0]), nick);
+						   parv[0]), "Duplicate",nick);
 				continue;
 			    }
 		    }
@@ -1126,7 +1161,7 @@ int	notice;
 		    }
 		sendto_one(sptr, err_str(ERR_NOSUCHNICK, parv[0]), nick);
 	    }
-    return nickcnt;
+    return penalty;
 }
 
 /*
@@ -1159,9 +1194,10 @@ char	*parv[];
 	return m_message(cptr, sptr, parc, parv, 1);
 }
 
-static	void	do_who(sptr, acptr, repchan)
+static	void	do_who(sptr, acptr, repchan, lp)
 aClient *sptr, *acptr;
 aChannel *repchan;
+Link *lp;
 {
 	char	status[5];
 	int	i = 0;
@@ -1172,10 +1208,15 @@ aChannel *repchan;
 		status[i++] = 'H';
 	if (IsAnOper(acptr))
 		status[i++] = '*';
-	if (repchan && is_chan_op(acptr, repchan))
-		status[i++] = '@';
-	else if (repchan && has_voice(acptr, repchan))
-		status[i++] = '+';
+	if ((repchan != NULL) && (lp == NULL))
+		lp = find_user_link(repchan->members, acptr);
+	if (lp != NULL)
+	    {
+		if (lp->flags & CHFL_CHANOP)
+			status[i++] = '@';
+		else if (lp->flags & CHFL_VOICE)
+			status[i++] = '+';
+	    }
 	status[i] = '\0';
 	sendto_one(sptr, rpl_str(RPL_WHOREPLY, sptr->name),
 		   (repchan) ? (repchan->chname) : "*", acptr->user->username,
@@ -1185,14 +1226,14 @@ aChannel *repchan;
 
 
 /*
-** m_who
+** r_who
 **	parv[0] = sender prefix
 **	parv[1] = nickname mask list
 **	parv[2] = additional selection flag, only 'o' for now.
 */
-int	m_who(cptr, sptr, parc, parv)
+int	m_who(cptr, sptr, parc, parv, depth)
 aClient *cptr, *sptr;
-int	parc;
+int	parc, depth;
 char	*parv[];
 {
 	Reg	aClient *acptr;
@@ -1200,17 +1241,14 @@ char	*parv[];
 	Reg	Link	*lp;
 	aChannel *chptr;
 	aChannel *mychannel;
-	char	*channame = NULL, *s;
+	char	*channame = NULL, *s = NULL;
 	int	oper = parc > 2 ? (*parv[2] == 'o' ): 0; /* Show OPERS only */
 	int	member, penalty = 0;
 
 	if (!BadPtr(mask))
 	    {
 		if ((s = (char *)index(mask, ',')))
-		    {
-			parv[1] = ++s;
-			penalty = m_who(cptr, sptr, parc, parv);
-		    }
+			*s = '\0';
 		clean_channelname(mask);
 	    }
 
@@ -1256,14 +1294,14 @@ char	*parv[];
 					if (IsInvisible(lp->value.cptr) &&
 					    !member)
 						continue;
-					do_who(sptr, lp->value.cptr, chptr);
+					do_who(sptr, lp->value.cptr, chptr,lp);
 				    }
 			penalty += 1;
 		    }
 		else if (chptr && IsAnonymous(chptr) &&
 			 (lp = find_user_link(chptr->members, sptr)))
 		    {
-			do_who(sptr, lp->value.cptr, chptr);
+			do_who(sptr, lp->value.cptr, chptr, lp);
 			penalty += 1;
 		    }
 	    }
@@ -1322,12 +1360,18 @@ char	*parv[];
 			     match(mask, acptr->user->host) == 0 ||
 			     match(mask, acptr->user->server) == 0 ||
 			     match(mask, acptr->info) == 0))
-				do_who(sptr, acptr, ch2ptr);
+				do_who(sptr, acptr, ch2ptr, NULL);
 		    }
 		penalty += 3;
 	    }
 	sendto_one(sptr, rpl_str(RPL_ENDOFWHO, parv[0]),
 		   BadPtr(mask) ?  "*" : mask);
+	if (s)
+	    {
+		parv[1] = ++s;
+		if (penalty < MAXPENALTY)
+			penalty += m_who(cptr, sptr, parc, parv);
+	    }
 	return penalty;
 }
 
@@ -1375,7 +1419,7 @@ char	*parv[];
 	    {
 		if (hunt_server(cptr,sptr,":%s WHOIS %s :%s", 1,parc,parv) !=
 		    HUNTED_ISME)
-			return 2;
+			return 3;
 		parv[1] = parv[2];
 	    }
 
@@ -1512,7 +1556,7 @@ char	*parv[];
 	    }
 	sendto_one(sptr, rpl_str(RPL_ENDOFWHOIS, parv[0]), parv[1]);
 
-	return 1;
+	return 2;
 }
 
 /*
@@ -1572,16 +1616,28 @@ char	*parv[];
 		else
 		    {
 			if (!(sp = find_tokserver(atoi(server), cptr, NULL)))
-				acptr = find_server(server, NULL); /* hmm??? */
-/*			strncpyzt(user->tok, server, sizeof(user->tok));*/
+			    {
+				/*
+				** Why? Why do we keep doing this?
+				** s_service.c had the same kind of kludge.
+				** Can't we get rid of this? - krys
+				*/
+				acptr = find_server(server, NULL);
+				if (acptr)
+					sendto_flag(SCH_ERROR,
+			    "ERROR: SERVER:%s uses wrong syntax for NICK (%s)",
+					    get_client_name(cptr, FALSE),
+						    parv[0]);
+			    }
 		    }
 		if (acptr)
 			sp = acptr->serv;
 		else if (!sp)
 		    {
 			sendto_flag(SCH_ERROR,
-                        	    "ERROR: USER:%s without SERVER:%s",
-				    parv[0], server);
+                        	    "ERROR: USER:%s without SERVER:%s from %s",
+				    parv[0], server,
+				    get_client_name(cptr, FALSE));
 			ircstp->is_nosrv++;
 			return exit_client(NULL, sptr, &me, "No Such Server");
 		    }
@@ -1591,11 +1647,7 @@ char	*parv[];
 		Debug((DEBUG_DEBUG, "from %s user %s server %s -> %#x %s",
 			parv[0], username, server, sp, sp->bcptr->name));
 		strncpyzt(user->host, host, sizeof(user->host));
-#ifndef KRYS
-		strcpy(user->server, sp->bcptr->name);
-#else
 		user->server = find_server_string(sp->snum);
-#endif
 		goto user_finish;
 	    }
 
@@ -1608,11 +1660,7 @@ char	*parv[];
 		sptr->user->flags |= FLAGS_RESTRICTED;
 	sptr->user->flags |= (UFLAGS & atoi(host));
 	strncpyzt(user->host, host, sizeof(user->host));
-#ifndef KRYS
-	strcpy(user->server, ME);
-#else
 	user->server = find_server_string(me.serv->snum);
-#endif
 
 user_finish:
 	/* 
@@ -1663,7 +1711,7 @@ user_finish:
 	    }
 	else
 		strncpyzt(sptr->user->username, username, USERLEN+1);
-	return 0;
+	return 2;
 }
 
 /*
@@ -1754,7 +1802,7 @@ char	*parv[];
 			    sptr->name, acptr->name, parv[0], parv[1], parv[2]);
 		sendto_one(sptr, err_str(ERR_CANTKILLSERVER, parv[0]),
 			   acptr->name);
-		return 0;
+		return 1;
 	    }
 
 #ifdef	LOCAL_KILL_ONLY
@@ -1821,7 +1869,7 @@ char	*parv[];
 	    {
 		sendto_serv_butone(cptr, ":%s KILL %s :%s!%s",
 				   parv[0], acptr->name, inpath, path);
-		if (chasing)
+		if (chasing && !IsClient(cptr))
 			sendto_one(cptr, ":%s KILL %s :%s!%s",
 				   ME, acptr->name, inpath, path);
 		acptr->flags |= FLAGS_KILLED;
@@ -1945,7 +1993,7 @@ char	*parv[];
 	sptr->user->away = away;
 	(void)strcpy(away, awy2);
 	sendto_one(sptr, rpl_str(RPL_NOWAWAY, parv[0]));
-	return 1;
+	return 2;
 }
 
 /*
@@ -2021,22 +2069,21 @@ char	*parv[];
 	if (!BadPtr(destination) && mycmp(destination, ME) != 0)
 	    {
 		if ((acptr = find_client(destination, NULL)) ||
-		    (acptr = find_server(destination, NULL)))
-			sendto_one(acptr,":%s PONG %s %s",
-				   parv[0], origin, destination);
-		else
-		    {
+		    (acptr = find_server(destination, NULL))) {
+			if (!(MyClient(sptr) && mycmp(origin, sptr->name)))
+				sendto_one(acptr,":%s PONG %s %s",
+					   parv[0], origin, destination);
+		} else
 			sendto_one(sptr, err_str(ERR_NOSUCHSERVER, parv[0]),
 				   destination);
-			return 1;
-		    }
+		return 2;
 	    }
 #ifdef	DEBUGMODE
 	else
 		Debug((DEBUG_NOTICE, "PONG: %s %s", origin,
 		      destination ? destination : "*"));
 #endif
-	return 0;
+	return 1;
     }
 
 
@@ -2071,6 +2118,7 @@ char	*parv[];
 	    
 	if ((IsServer(cptr) || IsMe(cptr)) && !IsOper(sptr))
 	    {
+		/* we never get here, do we?? (counters!) -krys */
 		sptr->user->flags |= FLAGS_OPER;
 		sendto_serv_butone(cptr, ":%s MODE %s :+o", parv[0], parv[0]);
 		if (IsMe(cptr))
@@ -2128,7 +2176,7 @@ char	*parv[];
 #ifdef	OPER_REMOTE
 		if (aconf->status == CONF_LOCOP)
 #else
-		if ((matches(s,me.sockhost) && !IsLocal(sptr)) ||
+		if ((match(s,me.sockhost) && !IsLocal(sptr)) ||
 		    aconf->status == CONF_LOCOP)
 #endif
 			SetLocOp(sptr);
@@ -2195,7 +2243,7 @@ char	*parv[];
 		(void)detach_conf(sptr, aconf);
 		sendto_one(sptr,err_str(ERR_PASSWDMISMATCH, parv[0]));
 	    }
-	return 1;
+	return 3;
     }
 
 /***************************************************************************
@@ -2207,6 +2255,8 @@ char	*parv[];
 **	parv[0] = sender prefix
 **	parv[1] = password
 **	parv[2] = version (server only)
+**	parv[3] = server options (server only)                              
+**	parv[4] = (optional) link options (server only)                  
 */
 int	m_pass(cptr, sptr, parc, parv)
 aClient *cptr, *sptr;
@@ -2220,10 +2270,20 @@ char	*parv[];
 		sendto_one(cptr, err_str(ERR_NEEDMOREPARAMS, parv[0]), "PASS");
 		return 1;
 	    }
-	if (parc > 2 && parv[2])
-		strncpyzt(cptr->info, parv[2], NICKLEN);
+	/* Temporarily store PASS pwd *parameters* into info field */
+	if (parc > 2 && parv[2]) {
+		strncpyzt(cptr->info, parv[2], 11); 
+		if (parc > 3 && parv[3]) {
+			strncpy(cptr->info+12, parv[3], 30);
+			cptr->info[42] = '\0';
+			if (parc > 4 && parv[4]) {
+				strncpy(cptr->info+44, parv[4], 5);
+				cptr->info[49] = '\0';
+			}
+		}
+	}
 	strncpyzt(cptr->passwd, password, sizeof(cptr->passwd));
-	return 0;
+	return 1;
     }
 
 /*
@@ -2325,54 +2385,6 @@ char	*parv[];
 	return 1;
 }
 
-#if !defined(NPATH)
-int	m_note(cptr, sptr, parc, parv)
-aClient	*cptr, *sptr;
-int	parc;
-char	*parv[];
-{
-	Reg	aServer *asptr;
-	Reg	int	i = 0;
-	aClient	*acptr;
-	int	wilds = 0;
-	char	*c, nbuf[50];
-
-	if (parc < 2)
-		return 1;
-
-	c = parv[1];
-
-	while (*c && *c != ' ' && i < 49)
-	    {
-		if (*c == '*' || *c == '?')
-			wilds = 1;
-	  	nbuf[i++] = *c++;
-	    }
-
-	nbuf[i] = 0;
-
-	if (wilds && (IsOper(sptr) || IsServer(sptr)))
-		for (i = highest_fd; i >= 0; i--)
-		    {
-			if (!(acptr = local[i]))
-				continue;
-			if (IsServer(acptr) && acptr != cptr)
-				sendto_one(acptr, ":%s NOTE :%s",
-					   parv[0], parv[1]);
-		    }
-	else
-		for (asptr = svrtop; asptr; asptr = asptr->nexts)
-			if ((acptr = asptr->bcptr) && acptr != cptr &&
-			    !mycmp(nbuf, acptr->name))
-			    {
-				sendto_one(acptr, ":%s NOTE :%s",
-					   parv[0], parv[1]);
-				break;
-			    }
-	return 1;
-}
-#endif
-
 static int user_modes[]	     = { FLAGS_OPER, 'o',
 				 FLAGS_LOCOP, 'O',
 				 FLAGS_INVISIBLE, 'i',
@@ -2477,6 +2489,7 @@ char	*parv[];
 						sptr->user->flags |= flag;
 					else
 						sptr->user->flags &= ~flag;	
+					penalty += 1;
 					break;
 				    }
 				if (flag == 0 && MyConnect(sptr))
@@ -2515,13 +2528,11 @@ char	*parv[];
 		    {
 			istat.is_user[1]--;
 			istat.is_user[0]++;
-			penalty += 1;
 		    }
 		if (IsInvisible(sptr) && !(setflags & FLAGS_INVISIBLE))
 		    {
 			istat.is_user[1]++;
 			istat.is_user[0]--;
-			penalty += 1;
 		    }
 		send_umode_out(cptr, sptr, setflags);
 	    }
@@ -2633,7 +2644,9 @@ int	old;
 		send_umode(cptr, sptr, old, ALL_UMODES, buf);
 #ifdef USE_SERVICES
 	/* buf contains all modes for local users, and iow only for remotes */
-	check_services_butone(SERVICE_WANT_UMODE, NULL, sptr,
-			      ":%s MODE %s :%s", sptr->name, sptr->name, buf);
+	if (*buf)
+		check_services_butone(SERVICE_WANT_UMODE, NULL, sptr,
+				      ":%s MODE %s :%s", sptr->name,
+				      sptr->name, buf);
 #endif
 }
